! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: ims_ascregion_mod
!
! Authors:  Ian Stewart (SSC/LUX)
!          
! Description: Replacement for mssllib/src/ascregion_mod.
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
module ims_ascregion_mod

  use dal
  use errorHandling

  use wcs_aux ! has typedef for wcs structure.

  implicit none 

  private 

!  public writeRegionFile
  public initializeRegionT
  public copyRegionT
  public initializeRegionExtnT
  public copyRegionExtnT
  public writeRegionExtension

  interface initializeRegionT
    module procedure initializeRegionTScalar
    module procedure initializeRegionTVector
  end interface

  interface initializeRegionExtnT
    module procedure initializeRegionExtnTScalar
    module procedure initializeRegionExtnTVector
  end interface

  interface writeRegionExtension
    module procedure writeRegionExtensionScalar
    module procedure writeRegionExtensionVector
  end interface

  integer, parameter :: mySHAPE_LEN = 16
  integer, parameter :: myMAX_POLY = 4  !!!!!50

  type, public :: ims_RegionT
    character(mySHAPE_LEN) ::&
      shape
    real(single) ::&
      x(myMAX_POLY),&
      y(myMAX_POLY),&
      r(myMAX_POLY),&
      a(myMAX_POLY)
    integer(int32) ::&
      component
  end type ims_RegionT

  type, public :: ims_RegionExtnT
    character(8) ::&
      extName
    character(20) ::&
      xColName,&
      yColName
    logical(bool) ::&
      withWcs
    type(WcsT) ::&
      wcs
    type(ims_RegionT), pointer ::&
      regions(:)
  end type ims_RegionExtnT

contains 
  
  !---------------------------------------------------------------------
  function copyRegionT(sourceRegion) result(destinationRegion)
  !---------------------------------------------------------------------

    implicit none

    type(ims_RegionT), intent(in) :: sourceRegion
    type(ims_RegionT) :: destinationRegion

    !-------------------------------------------------------------------
    ! Start

    destinationRegion%shape     = sourceRegion%shape
    destinationRegion%x         = sourceRegion%x
    destinationRegion%y         = sourceRegion%y
    destinationRegion%r         = sourceRegion%r
    destinationRegion%a         = sourceRegion%a
    destinationRegion%component = sourceRegion%component

  end function copyRegionT

  !---------------------------------------------------------------------
  function copyRegionExtnT(sourceRegionExtn) result(destinationRegionExtn)
  !---------------------------------------------------------------------
  ! NOTE! The calling program should deallocate the pointer
  ! destinationRegionExtn%regions after use.

    implicit none

    type(ims_RegionExtnT), intent(in) :: sourceRegionExtn
    type(ims_RegionExtnT) :: destinationRegionExtn

    integer :: numRegs, regNum

    !-------------------------------------------------------------------
    ! Start

    destinationRegionExtn%extName  = sourceRegionExtn%extName
    destinationRegionExtn%xColName = sourceRegionExtn%xColName
    destinationRegionExtn%yColName = sourceRegionExtn%yColName
    destinationRegionExtn%withWcs  = sourceRegionExtn%withWcs
    destinationRegionExtn%wcs      = copyWcsT(sourceRegionExtn%wcs)

    numRegs = size(sourceRegionExtn%regions)
    allocate(destinationRegionExtn%regions(numRegs))
    do regNum = 1, numRegs
      destinationRegionExtn%regions(regNum)&
        = copyRegionT(sourceRegionExtn%regions(regNum))
    end do

  end function copyRegionExtnT

  !---------------------------------------------------------------------
  subroutine initializeRegionTScalar(region)
  !---------------------------------------------------------------------

    implicit none

    type(ims_RegionT), intent(out) :: region

    type(ims_RegionT) :: regionVec(1)

    !-------------------------------------------------------------------
    ! Start

    call initializeRegionTVector(regionVec)
    region = copyRegionT(regionVec(1))

  end subroutine initializeRegionTScalar

  !---------------------------------------------------------------------
  subroutine initializeRegionTVector(regions)
  !---------------------------------------------------------------------

    implicit none

    type(ims_RegionT), intent(out) :: regions(:)

    integer :: numRegions, regionNum

    !-------------------------------------------------------------------
    ! Start

    numRegions = size(regions)
    do regionNum = 1, numRegions
      regions(regionNum)%shape = 'POINT'
      regions(regionNum)%x = 0.0
      regions(regionNum)%y = 0.0
      regions(regionNum)%r = 0.0
      regions(regionNum)%a = 0.0
      regions(regionNum)%component = 1
    end do

  end subroutine initializeRegionTVector

  !---------------------------------------------------------------------
  subroutine initializeRegionExtnTScalar(regionExtn)
  !---------------------------------------------------------------------

    implicit none

    type(ims_RegionExtnT), intent(out) :: regionExtn

    type(ims_RegionExtnT) :: regionExtnVec(1)

    !-------------------------------------------------------------------
    ! Start

    call initializeRegionExtnTVector(regionExtnVec)
    regionExtn = copyRegionExtnT(regionExtnVec(1))

  end subroutine initializeRegionExtnTScalar

  !---------------------------------------------------------------------
  subroutine initializeRegionExtnTVector(regionExtnData)
  !---------------------------------------------------------------------

    implicit none

    type(ims_RegionExtnT), intent(out) :: regionExtnData(:)

    integer :: numExtensions, extnNum

    !-------------------------------------------------------------------
    ! Start

    numExtensions = size(regionExtnData)

    regionExtnData%extName = ''
    regionExtnData%xColName = ''
    regionExtnData%yColName = ''
    regionExtnData%withWcs = .false.

    do extnNum = 1, numExtensions
      call initializeWcsT(regionExtnData(extnNum)%wcs) ! in ssclib/wcs_aux
    end do

!    allocate(regionExtn%regions(0))
! Not sure it make sense to allocate this, even to zero.

  end subroutine initializeRegionExtnTVector

  !---------------------------------------------------------------------
  subroutine writeRegionExtensionScalar(regionSetName, regionExtnData)
  !---------------------------------------------------------------------

    implicit none

    character(*), intent(in) :: regionSetName
    type(ims_RegionExtnT), intent(in) :: regionExtnData

    type(ims_RegionExtnT) :: regionExtnDataVec(1)

    !-------------------------------------------------------------------
    ! Start

    regionExtnDataVec(1) = copyRegionExtnT(regionExtnData)
    call writeRegionExtensionVector(regionSetName, regionExtnDataVec)
    deallocate(regionExtnDataVec(1)%regions)

  end subroutine writeRegionExtensionScalar

  !---------------------------------------------------------------------
  subroutine writeRegionExtensionVector(regionSetName, regionExtnData)
  !---------------------------------------------------------------------

    implicit none

    character(*), intent(in) :: regionSetName
    type(ims_RegionExtnT), intent(in) :: regionExtnData(:)

    integer :: numExtensions, extNum, dims(0:0), stringDims(0:0), numRows, row
    type(DataSetT) :: set
    type(TableT) :: tab
    character(20) :: xColName, yColName
    type(ColumnT) :: shapeCol, xCol, yCol, rCol, aCol, compCol
    real(single), dimension(:,:), pointer :: xData, yData, rData, aData
    integer(int32), pointer :: compData(:,:)

    !-------------------------------------------------------------------
    ! Start

    numExtensions = size(regionExtnData)

    dims(0) = myMAX_POLY
    stringDims(0) = mySHAPE_LEN

    set = dataSet(regionSetName, CREATE)

    do extNum = 1, numExtensions
      numRows = size(regionExtnData(extNum)%regions)
      tab = addTable(set, regionExtnData(extNum)%extName, numRows)

      ! Write HDUCLAS keywords, etc.

      xColName = regionExtnData(extNum)%xColName	! Just for
      yColName = regionExtnData(extNum)%yColName	! convenience.

      call setAttribute(tab, "HDUVERS", "1.0.0")
      call setAttribute(tab, "HDUCLASS", "ASC")
      call setAttribute(tab, "HDUCLAS1", "REGION")
      call setAttribute(tab, "HDUCLAS2", "STANDARD")
      call setAttribute(tab, "MTYPE1", "pos")
      call setAttribute(tab, "MFORM1", trim(xColName)//","//trim(yColName))

      shapeCol = addColumn(tab, "SHAPE",        STRING, " ", stringDims)
      xCol     = addColumn(tab, trim(xColName), REAL32, " ", dims)
      yCol     = addColumn(tab, trim(yColName), REAL32, " ", dims)
      rCol     = addColumn(tab, "R",            REAL32, " ", dims)
      aCol     = addColumn(tab, "ROTANG",       REAL32, " ", dims)
      compCol  = addColumn(tab, "COMPONENT",    INTEGER32)

      xData     => real32VectorData(xCol)
      yData     => real32VectorData(yCol)
      rData     => real32VectorData(rCol)
      aData     => real32VectorData(aCol)
      compData  => int32VectorData(compCol)

!      xData    = regionExtnData(extNum)%regions%x
!      yData    = regionExtnData(extNum)%regions%y
!      rData    = regionExtnData(extNum)%regions%r
!      aData    = regionExtnData(extNum)%regions%a
!      compData = regionExtnData(extNum)%regions%component

      do row = 0, numRows - 1
        call setStringCell(shapeCol, row&
          , regionExtnData(extNum)%regions(row+1)%shape)

! DON'T trim the string argument to setStringCell. If you do, the characters
! in the file that should be trailing blanks will not be overwritten from
! whatever they were before in the memory map of the file, and will thus
! contain nonsense characters.

        xData(:, row)    = regionExtnData(extNum)%regions(row+1)%x
        yData(:, row)    = regionExtnData(extNum)%regions(row+1)%y
        rData(:, row)    = regionExtnData(extNum)%regions(row+1)%r
        aData(:, row)    = regionExtnData(extNum)%regions(row+1)%a
        compData(:, row) = regionExtnData(extNum)%regions(row+1)%component
      end do

      if (regionExtnData(extNum)%withWcs) then
!        call setAttribute(xCol, "TCTYP", regionExtnData(extNum)%wcs%X%ctype)
!        call setAttribute(xCol, "TCRPX", regionExtnData(extNum)%wcs%X%refPixels)
!        call setAttribute(xCol, "TCRVL", regionExtnData(extNum)%wcs%X%refWorld)
!        call setAttribute(xCol, "TCDLT"&
!          , regionExtnData(extNum)%wcs%X%pixelSizeWorld)
!
!        call setAttribute(yCol, "TCTYP", regionExtnData(extNum)%wcs%Y%ctype)
!        call setAttribute(yCol, "TCRPX", regionExtnData(extNum)%wcs%Y%refPixels)
!        call setAttribute(yCol, "TCRVL", regionExtnData(extNum)%wcs%Y%refWorld)
!        call setAttribute(yCol, "TCDLT"&
!          , regionExtnData(extNum)%wcs%Y%pixelSizeWorld)

call writeWcs(xCol, yCol, regionExtnData(extNum)%wcs) ! in ssclib/wcs-aux
      end if

      call release(tab)
    end do

    call release(set)

  end subroutine writeRegionExtensionVector

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module ims_ascregion_mod
