! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program mkevents

  use dal

  implicit none

  ! Fake pointing direction
  real (kind=double), parameter :: pntRA=193.18543957
  real (kind=double), parameter :: pntDec=-18.235478234
  real (kind=double), parameter :: pntApos =  32.331659821

  ! Some size (in arcseconds)
  real (kind=single), parameter :: ccdPixelSize=4
  real (kind=single), parameter :: skyPixelSize=0.05
  real (kind=single), parameter :: fieldOfView=17.5*60
  
  ! Handles
  type(DataSetT) :: set
  type(TableT) :: tab
  type(ColumnT) :: xcol, ycol

  ! Pointers, etc.
  integer :: i, hw, p, q, nevt
  integer, pointer, dimension(:) :: x, y

  ! Create dataset
  hw=int(fieldOfView/ccdPixelSize)
  nevt=4*(hw+1)**2
  set=dataSet("events.fits",create)
  tab=addTable(set,"EVENTS",nevt)

  call setAttribute(set, "INSTRUME", "EMOS1")
  call setAttribute(set, "RA_PNT",   pntRA)
  call setAttribute(set, "DEC_PNT",  pntDec)
  call setAttribute(set, "PA_PNT",   pntApos)
  call setAttribute(set, "DATE-OBS", '2001-11-27T04:31:29')

  ! Add columns
  xcol=addColumn(tab,"X",Integer32,"0.05 arcsec")
  ycol=addColumn(tab,"Y",Integer32,"0.05 arcsec")
  x=>int32Data(xcol)
  y=>int32Data(ycol)

  ! Populate
  i=0
  do q=-hw, hw+1
     do p=-hw, hw+1
        x(i)=nint(p*ccdPixelSize/skyPixelSize)
        y(i)=nint(q*ccdPixelSize/skyPixelSize)
        i=i+1
     end do
  end do

  ! Add limits, WCS keywords, etc.
  call setAttribute(xcol,"TLMIN",-hw*ccdPixelSize/skyPixelSize)
  call setAttribute(xcol,"TLMAX",hw*ccdPixelSize/skyPixelSize)
  call setAttribute(xcol,"TCDLT",-skyPixelSize/3600.0)
  call setAttribute(xcol,"TCRVL",pntRA)
  call setAttribute(xcol,"TCRPX",0)
  call setAttribute(xcol,"TCTYP","RA---TAN")

  call setAttribute(ycol,"TLMIN",-hw*ccdPixelSize/skyPixelSize)
  call setAttribute(ycol,"TLMAX",hw*ccdPixelSize/skyPixelSize)
  call setAttribute(ycol,"TCDLT",skyPixelSize/3600.0)
  call setAttribute(ycol,"TCRVL",pntDec)
  call setAttribute(ycol,"TCRPX",0)
  call setAttribute(ycol,"TCTYP","DEC--TAN")

  ! Release handles
  call release(xcol)
  call release(ycol)
  call release(tab)
  call release(set)

end program mkevents
