! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program testingestsrcnamesaux
  ! Authors:  Masaaki Sakano - SSC/LUX
  ! NOTE: 

  use errorhandling
  use types
  use dal		! 

  use region_aux

  ! Followings are in ssclib
  !use constants		! for Pi
  use test_utils	! for isNearlyEqual()

  implicit none

  character(36), parameter :: &
       OUTFILNAME  = "outfiles/testinput_for_getstrs.fits" &
     , OUTFILNAME2 = "outfiles/testinput_for_retkwd2.fits" &
     , OUTFILNAME3 = "outfiles/testinput_for_retkwd3.fits"
  character(80), parameter :: &
       INCATFILENAME = "testdata/testcat.fits"
  character(10), parameter :: TMPTBLNAME = "test_table"

  integer, parameter :: MAINDEFARRAYSIZE = 3
  real(single), dimension(MAINDEFARRAYSIZE) :: srcRadEffExample
  real(single), dimension(MAINDEFARRAYSIZE, MAINDEFARRAYSIZE) :: &
       sepExample, oveFracExample

  !-------------------------------------------------------------------
  ! Start:

  call test_setOverlapFractions
  call test_calcSepAndOverlapFrac(srcRadEffExample, sepExample, oveFracExample)	! Get those values.
  call test_findMinPosMaxOverlap(oveFracExample)	! Pass oveFracExample(:,:)
    ! iOfMinMax => 1, jOfMinMax => 2  (See the above test_findMinPosMaxOverlap())
  call test_shrinkRadiusEffective(srcRadEffExample, sepExample)	! Pass these.
  call test_adjustOverlaps()	! Pass these.
  ! call test_adjustOverlaps(srcRadEffExample, sepExample, oveFracExample)	! Pass these.

contains

  !---------------------------------------------------------------------
  subroutine test_setOverlapFractions()
  !---------------------------------------------------------------------
  ! Tests of setOverlapFractions()
  ! NOTE: 

    implicit none

    character(28)  :: errHead
    integer, parameter :: defArraySize = MAINDEFARRAYSIZE

    real(single), dimension(defArraySize) :: srcRadiusEffective
    real(single), dimension(defArraySize,defArraySize) :: &
       separations, overlapFraction

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    ! setOverlapFractions()
    !----------------------------------------------------------
    errHead = "setOverlapFractions001a"
    print *,"=== "//trim(errHead)//": Simple"

    ! Prepare a set of example test data
    srcRadiusEffective(:) = (/ 4, 1, 3 /)
    separations(1,:) = (/ 0, 1, 2 /)
    separations(3,:) = (/ 2, 1, 0 /)

    call setOverlapFractions(srcRadiusEffective &
       , separations, overlapFraction, 1, 3)

    !------ MEMO -------:
    ! sumOfRadii == 4 + 3 = 7
    ! overlap == sumOfRadii - 2 = 5
    !  overlapFraction(1,3) => overlap/4 = 1.25
    !  overlapFraction(3,1) => overlap/3 = 1.66666666667

    if (    (.not. isNearlyEqual(overlapFraction(1,3), 5.0/4, 6)) &
       .or. (.not. isNearlyEqual(overlapFraction(3,1), 5.0/3, 6)) ) then
      print *, 'overlapFraction(1,3) is ', overlapFraction(1,3), ' which should be ', 5.0/4
      print *, 'overlapFraction(3,1) is ', overlapFraction(3,1), ' which should be ', 5.0/3
      call error("err"//trim(errHead), '')
    end if

  end subroutine test_setOverlapFractions


  !---------------------------------------------------------------------
  subroutine test_calcSepAndOverlapFrac(srcRadiusEffectiveOut, sepExpectedOut, oveFracExpectedOut)
  !---------------------------------------------------------------------
  ! Tests of calcSepAndOverlapFrac()
  ! NOTE: 

    implicit none

    character(28)  :: errHead
    integer, parameter :: defArraySize = MAINDEFARRAYSIZE

    real(single), dimension(defArraySize) :: srcX, srcY
    real(single), dimension(defArraySize) :: srcRadiusEffective
    real(single), dimension(defArraySize), intent(out), optional :: srcRadiusEffectiveOut
    real(single), dimension(defArraySize,defArraySize) :: &
       separations,    overlapFraction
    real(single), dimension(defArraySize,defArraySize) :: &
       sepExpected,    oveFracExpected
    real(single), dimension(defArraySize,defArraySize), intent(out), optional :: &
       sepExpectedOut, oveFracExpectedOut
    integer :: i, j
    logical :: isFine

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    ! calcSepAndOverlapFrac()
    !----------------------------------------------------------
    errHead = "calcSepAndOverlapFrac001a"
    print *,"=== "//trim(errHead)//": Simple (separations-symmetry)"

    srcX(:) = (/ 1, 1, -1 /)
    srcY(:) = (/ 2, 4, -2 /)
    srcRadiusEffective(:) = (/ 4, 1, 3 /)

    call calcSepAndOverlapFrac(srcX, srcY, srcRadiusEffective &
       , separations, overlapFraction)

    ! Expected returned values:
    sepExpected(1,:) = (/   0.0,       2.0,   sqrt(20.0) /)	! 2 = 4-2
    sepExpected(2,:) = (/   2.0,       0.0,   sqrt(40.0) /)	! sqrt(40=2^2+6^2) => 6.32455532033676
    sepExpected(3,:) = (/ sqrt(20.0), sqrt(40.0), 0.0    /)	! sqrt(20=2^2+4^2) => 4.47213595499958

    oveFracExpected(1,:) = (/   0.0,             0.75, (7-sqrt(20.0))/4 /)	! 0.75 = ((4+1)-2)/4, ((7=4+3)-sqrt(20))/4 => 0.631966011250105
    oveFracExpected(2,:) = (/   3.0,             0.0,  (4-sqrt(40.0))   /)	! 3    = ((4+1)-2)/1, ((4=1+3)-sqrt(40))/1 => -2.32455532033676
    oveFracExpected(3,:) = (/ (7-sqrt(20.0))/3, (4-sqrt(40.0))/3,  0.0  /)	! ((7=4+3)-sqrt(20))/3 => 0.842621348, ((4=1+3)-sqrt(40))/3 => -0.774851773445586

    ! Check the symmetry in separations
    isFine = .true.
    do i=1, defArraySize
      do j=i, defArraySize
        if (separations(i,j) .ne. separations(j,i)) isFine = .false.
      end do
    end do

    if ( .not. isFine) then
       print *, 'separations(:) =', separations
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "calcSepAndOverlapFrac002a"
    print *,"=== "//trim(errHead)//": Thorough (separations)"

    ! Check the entire matrix of separations(:,:).
    ! See "calcSepAndOverlapFrac001a" for the expected values.
    isFine = .true.
    do i=1, defArraySize
      do j=1, defArraySize
        if (.not. isNearlyEqual(separations(i,j), sepExpected(i,j), 6)) isFine = .false.	! in ssclib/test_utils
      end do
    end do

    if ( .not. isFine) then
       print *, 'separations(:,:) ='
       print *, separations
       print *, 'sepExpected(:,:) ='
       print *, sepExpected
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "calcSepAndOverlapFrac003a"
    print *,"=== "//trim(errHead)//": Thorough (overlapFraction)"

    ! Check the entire matrix of overlapFraction(:,:).
    ! See "calcSepAndOverlapFrac001a" for the expected values.
    isFine = .true.
    do i=1, defArraySize
      do j=1, defArraySize
        if (.not. isNearlyEqual(overlapFraction(i,j), oveFracExpected(i,j), 6)) isFine = .false.	! in ssclib/test_utils
      end do
    end do

    if ( .not. isFine) then
       print *, 'overlapFraction(:,:) ='
       print *, overlapFraction
       print *, 'oveFracExpected(:,:) ='
       print *, oveFracExpected
       call error("err"//trim(errHead), '')
    end if


    !------------- Postprocessing
    if (present(srcRadiusEffectiveOut)) srcRadiusEffectiveOut = srcRadiusEffective
    if (present(sepExpectedOut))     sepExpectedOut     = sepExpected
    if (present(oveFracExpectedOut)) oveFracExpectedOut = oveFracExpected

  end subroutine test_calcSepAndOverlapFrac


  !---------------------------------------------------------------------
  subroutine test_findMinPosMaxOverlap(overlapFractionIn)
  !---------------------------------------------------------------------
  ! Tests of findMinPosMaxOverlap()
  ! NOTE: 

    implicit none

    character(28)  :: errHead
    integer, parameter :: defArraySize = MAINDEFARRAYSIZE

    ! real(single)   :: srcRadiusEffective(defArraySize) &
    !                , separations(defArraySize,defArraySize) &
    real(single), intent(in), optional :: overlapFractionIn(defArraySize,defArraySize)
    real(single) :: overlapFraction(defArraySize,defArraySize)
    integer(int16) :: iOfMinMax, jOfMinMax
    logical :: noPositiveOverlaps

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    ! findMinPosMaxOverlap()
    !----------------------------------------------------------
    errHead = "FindMinPosMaxOverlap001a"
    print *,"=== "//trim(errHead)//": No overlap"

    overlapFraction(:,:) = 0

    call findMinPosMaxOverlap(overlapFraction, noPositiveOverlaps &
       , iOfMinMax, jOfMinMax)

    if (.not. noPositiveOverlaps) then
       print *, 'noPositiveOverlaps, iOfMinMax, jOfMinMax = ' &
          ,      noPositiveOverlaps, iOfMinMax, jOfMinMax
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "FindMinPosMaxOverlap002a"
    print *,"=== "//trim(errHead)//": Typical"

    ! See the comment-lines in region_aux.f90 for the meaning of the test data.
    overlapFraction(1,:) = (/ 0.0, 0.9, 3.0 /)
    overlapFraction(2,:) = (/ 2.3, 0.0, 0.4 /)
    overlapFraction(3,:) = (/ 0.8, 1.7, 0.0 /)

    call findMinPosMaxOverlap(overlapFraction, noPositiveOverlaps &
       , iOfMinMax, jOfMinMax)

    if (noPositiveOverlaps .or. iOfMinMax .ne. 3 .or. jOfMinMax .ne. 2) then
       print *, 'noPositiveOverlaps, iOfMinMax, jOfMinMax = ' &
          ,      noPositiveOverlaps, iOfMinMax, jOfMinMax
       call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
if (present(overlapFractionIn)) then
    errHead = "FindMinPosMaxOverlap010a"
    print *,"=== "//trim(errHead)//": Realistic"

    ! See the comment-lines in region_aux.f90 for the meaning of the test data.
    ! See oveFracExpected(:,:) in subroutine test_calcSepAndOverlapFrac for the test data here.

    call findMinPosMaxOverlap(overlapFractionIn, noPositiveOverlaps &
       , iOfMinMax, jOfMinMax)

    if (noPositiveOverlaps .or. iOfMinMax .ne. 1 .or. jOfMinMax .ne. 2) then
       print *, 'noPositiveOverlaps, iOfMinMax, jOfMinMax = ' &
          ,      noPositiveOverlaps, iOfMinMax, jOfMinMax
       call error("err"//trim(errHead), '')
    end if
end if	! if (present(overlapFractionIn)) then

    !----------------------------------------------------------
    ! errHead = "FindMinPosMaxOverlap003a"
    ! print *,"=== "//trim(errHead)//": Complicated"


  end subroutine test_findMinPosMaxOverlap


  !---------------------------------------------------------------------
  subroutine test_shrinkRadiusEffective(srcRadiusEffectiveIn, separationsIn)
  !---------------------------------------------------------------------
  ! Tests of shrinkRadiusEffective()
  ! NOTE: 

    implicit none

    character(28)  :: errHead
    integer, parameter :: defArraySize = MAINDEFARRAYSIZE

    real(single), intent(in) :: srcRadiusEffectiveIn(defArraySize) &
                    , separationsIn(defArraySize,defArraySize)
    real(single), dimension(defArraySize) :: &
       srcRadiusEffective, srcRadiusEffectiveMod
    real(single), dimension(defArraySize,defArraySize) :: &
       separations, overlapFraction
    integer(int16) :: iOfMinMax, jOfMinMax
    real(single) :: reduction

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    ! shrinkRadiusEffective()
    !----------------------------------------------------------
    errHead = "ShrinkRadiusEffective001a"
    print *,"=== "//trim(errHead)//": Simple"

    srcRadiusEffective(1:3) = (/ 3.0, 4.0, 2.0 /)
    separations(1,:)        = (/ 0.0, 2.5, 2.5 /)
    separations(2,:)        = (/ 2.5, 0.0, 2.5 /)
    separations(3,:)        = (/ 2.5, 2.5, 0.0 /)

    iOfMinMax=1
    jOfMinMax=2
    call shrinkRadiusEffective(srcRadiusEffective, separations&
       , iOfMinMax, jOfMinMax &
       , .true., overlapFraction, int(defArraySize, int16), reduction)

    if (.not. isNearlyEqual(reduction, 2.5/(3+4), 6)) then
      print *, 'reduction is ', reduction, ' which should be ', 2.5/(3+4)
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "ShrinkRadiusEffective011a"
    print *,"=== "//trim(errHead)//": Realistic(reduction)"
    ! See subroutine test_calcSepAndOverlapFrac() for the test data.

    srcRadiusEffectiveMod = srcRadiusEffectiveIn

    iOfMinMax = 1_int16	! See the MAIN for these figures.
    jOfMinMax = 2_int16
    call shrinkRadiusEffective(srcRadiusEffectiveMod, separationsIn&
       , iOfMinMax, jOfMinMax, reductionOut=reduction)

    !!! Extract from subroutine test_calcSepAndOverlapFrac()
    !! srcRadiusEffective(:) = (/ 4, 1, 3 /)
    !! sepExpected(1,:) = (/   0.0,       2.0,   sqrt(20.0) /)	! 2 = 4-2
    if ((.not. isNearlyEqual(reduction &
       , separationsIn(iOfMinMax,jOfMinMax)/(srcRadiusEffectiveIn(1)+srcRadiusEffectiveIn(2)) &
       , 6)) .or. &
        (.not. isNearlyEqual(reduction, 2.0/(4+1), 6)) ) then	! Identical as above, but just written discretely.
      print *, 'reduction is ', reduction, ' which should be ', 2.0/(4+1) 
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "ShrinkRadiusEffective012a"
    print *,"=== "//trim(errHead)//": Realistic(srcRadiusEffective)"
    ! See subroutine test_calcSepAndOverlapFrac() for the test data.

    !!! Extract from subroutine test_calcSepAndOverlapFrac()
    !! srcRadiusEffective(:) = (/ 4, 1, 3 /)
    !! sepExpected(1,:) = (/   0.0,       2.0,   sqrt(20.0) /)	! 2 = 4-2
    if (    (.not. isNearlyEqual(srcRadiusEffectiveMod(1), srcRadiusEffectiveIn(1)*reduction, 6)) &
       .or. (.not. isNearlyEqual(srcRadiusEffectiveMod(2), srcRadiusEffectiveIn(2)*reduction, 6)) &
       .or. (.not. isNearlyEqual(srcRadiusEffectiveMod(3), srcRadiusEffectiveIn(3),           6)) ) then
      print *, 'reduction = ', reduction
      print *, 'srcRadiusEffectiveIn  = ', srcRadiusEffectiveIn
      print *, 'srcRadiusEffectiveMod = ', srcRadiusEffectiveMod
      call error("err"//trim(errHead), '')
    end if

  end subroutine test_shrinkRadiusEffective

  !---------------------------------------------------------------------
  subroutine test_adjustOverlaps()
    ! srcRadiusEffectiveIn, separationsIn, overlapFractionIn)
  !---------------------------------------------------------------------
  ! Tests of adjustOverlaps()
  ! NOTE: 

    implicit none

    character(28)  :: errHead
    integer, parameter :: defArraySize = 6

    ! real(single), intent(in) :: srcRadiusEffectiveIn(:) &
    !                 , separationsIn(:,:) &
    !                 , overlapFractionIn(:,:)

    real(single), parameter :: defRealUninitialised = -999.0
    real(single), dimension(defArraySize) :: &
       srcRadiusEffective, expected
    real(single), dimension(defArraySize,defArraySize) :: &
       separations, overlapFraction, overlapFractionMod
    integer(int16) :: iOfMinMax
    ! integer :: i

    ! type(DataSetT) :: inSet
    ! type(TableT)   :: inTab
    ! integer(int32) :: obsId

    !-------------------------------------------------------------------
    ! Start:

    ! inSet = dataSet(OUTFILNAME, READ)
    ! inTab = table(inSet, 0)


    !----------------------------------------------------------
    ! adjustOverlaps()
    !----------------------------------------------------------
    errHead = "AdjustOverlaps001a"
    print *,"=== "//trim(errHead)//": Simple (unchanged values)"

    ! srcRadiusEffective(1:3) = (/ 3.0, 4.0, 2.0 /)
    separations(1,1:3)      = (/ 0.0, 2.0, 1.0 /)
    separations(2,1:3)      = (/ 2.0, 0.0, 1.0 /)
    separations(3,1:3)      = (/ 1.0, 1.0, 0.0 /)

    srcRadiusEffective(1:3) = (/ 3.0, 1.0, 1.0 /)

    overlapFraction  = defRealUninitialised

    iOfMinMax = 2

    call adjustOverlaps(srcRadiusEffective(1:3) &
       , separations(1:3,1:3), overlapFraction(1:3,1:3) &
       , iOfMinMax)

    if (    (.not. isNearlyEqual(overlapFraction(1,3), defRealUninitialised, 6)) &
      .or. (.not. isNearlyEqual(overlapFraction(2,2), defRealUninitialised, 6)) & 
      .or. (.not. isNearlyEqual(overlapFraction(3,1), defRealUninitialised, 6)) ) then
      print *,"overlapFraction() changed, which should not: ", overlapFraction
      call error("err"//trim(errHead), '')
    end if

    !----------------------------------------------------------
    errHead = "AdjustOverlaps002a"
    print *,"=== "//trim(errHead)//": Simple (changed values)"

    overlapFractionMod = overlapFraction
    call setOverlapFractions(srcRadiusEffective &
       , separations, overlapFractionMod, 1, 2)
    expected(1) = overlapFractionMod(1,2)
    expected(2) = overlapFractionMod(2,1)

    overlapFractionMod = overlapFraction
    call setOverlapFractions(srcRadiusEffective &
       , separations, overlapFractionMod, 2, 3)
    expected(3) = overlapFractionMod(2,3)
    expected(4) = overlapFractionMod(3,2)

    if      (.not. isNearlyEqual(overlapFraction(1,2), expected(1), 6)) then
      print *,'overlapFraction(1,2) is ', overlapFraction(1,2), 'which should be ', expected(1)
      call error("err"//trim(errHead), '')
    else if (.not. isNearlyEqual(overlapFraction(2,1), expected(2), 6)) then
      print *,'overlapFraction(2,1) is ', overlapFraction(1,2), 'which should be ', expected(2)
      call error("err"//trim(errHead), '')
    else if (.not. isNearlyEqual(overlapFraction(2,3), expected(3), 6)) then
      print *,'overlapFraction(2,3) is ', overlapFraction(2,3), 'which should be ', expected(3)
      call error("err"//trim(errHead), '')
    else if (.not. isNearlyEqual(overlapFraction(3,2), expected(4), 6)) then
      print *,'overlapFraction(3,2) is ', overlapFraction(3,2), 'which should be ', expected(4)
      call error("err"//trim(errHead), '')
    end if


    !----------------------------------------------------------
    errHead = "AdjustOverlaps003a"
    print *,"=== "//trim(errHead)//": Realistic"

    iOfMinMax = 2
    srcRadiusEffective    =(/ 6.0000000E+02, 2.4808350E+02, 2.9373630E+02, 5.8747626E+02, 3.4316183E+02, 1.0666254E+02/)
    separations( 1 ,:)    =(/ 0.0000000    , 1.5982455E+03, 1.1734453E+03, 2.2501831E+03, 1.6834824E+03, 1.2469691E+03/)
    separations( 2 ,:)    =(/ 1.5982455E+03, 0.0000000    , 6.5069513E+02, 1.7716586E+03, 8.4295984E+02, 3.5474603E+02/)
    separations( 3 ,:)    =(/ 1.1734453E+03, 6.5069513E+02, 0.0000000    , 2.1972161E+03, 1.3001464E+03, 4.0039883E+02/)
    separations( 4 ,:)    =(/ 2.2501831E+03, 1.7716586E+03, 2.1972161E+03, 0.0000000    , 9.3063812E+02, 1.8110247E+03/)
    separations( 5 ,:)    =(/ 1.6834824E+03, 8.4295984E+02, 1.3001464E+03, 9.3063812E+02, 0.0000000    , 9.0236835E+02/)
    separations( 6 ,:)    =(/ 1.2469691E+03, 3.5474603E+02, 4.0039883E+02, 1.8110247E+03, 9.0236835E+02, 0.0000000    /)
    overlapFraction( 1 ,:)=(/ 0.0000000, -1.2502700,     -0.4661817,  -1.7711780,    -1.2338676,     -0.9005110    /)
    overlapFraction( 2 ,:)=(/-3.0238287,  0.0000000,     -0.4388657,  -3.7733216,    -1.0146363,      3.0753334E-08/)
    overlapFraction( 3 ,:)=(/-0.9522453, -0.3706567,      0.0000000,  -4.4802213,    -2.2579715,      0.0000000    /)
    overlapFraction( 4 ,:)=(/-1.8089358, -1.5934241,     -2.2400966,   0.0000000,    -5.1946913E-08, -1.9011592    /)
    overlapFraction( 5 ,:)=(/-2.1573510, -0.7335154,     -1.9327564,  -8.8930570E-08, 0.0000000,     -1.3187480    /)
    overlapFraction( 6 ,:)=(/-5.0655699,  7.1528341E-08,  0.0000000, -10.4712095,    -4.2427635,      0.0000000    /)

    call adjustOverlaps(srcRadiusEffective, separations, overlapFraction, iOfMinMax)
    call adjustOverlaps(srcRadiusEffective, separations, overlapFraction, 6_int16)
    print *,'overlapFraction(2,6)=', overlapFraction(2,6)
    print *,'overlapFraction(3,6)=', overlapFraction(3,6)

    iOfMinMax = 3
    call adjustOverlaps(srcRadiusEffective, separations, overlapFraction, iOfMinMax)
    call adjustOverlaps(srcRadiusEffective, separations, overlapFraction, 6_int16)

    print *,'overlapFraction(2,6)=', overlapFraction(2,6)
    print *,'overlapFraction(3,6)=', overlapFraction(3,6)
    print *,'overlapFraction(:,:)=', overlapFraction

    ! obsId = getObsIdInt32FromHeader(inTab)
    ! if (obsId .ne. 234567891_int32) then
    !    print *,"ObsId from Tab is (",obsId,")"
    !    call error("err"//trim(errHead), '')
    ! end if


!  === After findMinPosMaxOverlap:  F   7.1528341E-08
!  Aft:iOfMinMax= 2  jOfMinMax= 6  overlapFrac(2,6)=   3.0753334E-08
!  Aft:iOfMinMax= 2  jOfMinMax= 6  overlapFrac(3,6)=   0.0000000
!  I:srcRadiusEffective(, 2 )=   2.4808350E+02  =>    2.4808350E+02
!  J:srcRadiusEffective(, 6 )=   1.0666254E+02  =>    1.0666254E+02
!  iOfMinMax = 2
!  srcRadiusEffective =   6.0000000E+02   2.4808350E+02   2.9373630E+02   5.8747626E+02   3.4316183E+02   1.0666254E+02
!  separations( 1 ,:)=   0.0000000   1.5982455E+03   1.1734453E+03   2.2501831E+03   1.6834824E+03   1.2469691E+03
!  separations( 2 ,:)=   1.5982455E+03   0.0000000   6.5069513E+02   1.7716586E+03   8.4295984E+02   3.5474603E+02
!  separations( 3 ,:)=   1.1734453E+03   6.5069513E+02   0.0000000   2.1972161E+03   1.3001464E+03   4.0039883E+02
!  separations( 4 ,:)=   2.2501831E+03   1.7716586E+03   2.1972161E+03   0.0000000   9.3063812E+02   1.8110247E+03
!  separations( 5 ,:)=   1.6834824E+03   8.4295984E+02   1.3001464E+03   9.3063812E+02   0.0000000   9.0236835E+02
!  separations( 6 ,:)=   1.2469691E+03   3.5474603E+02   4.0039883E+02   1.8110247E+03   9.0236835E+02   0.0000000
!  overlapFraction( 1 ,:)=   0.0000000  -1.2502700  -0.4661817  -1.7711780  -1.2338676  -0.9005110
!  overlapFraction( 2 ,:)=  -3.0238287   0.0000000  -0.4388657  -3.7733216  -1.0146363   3.0753334E-08
!  overlapFraction( 3 ,:)=  -0.9522453  -0.3706567   0.0000000  -4.4802213  -2.2579715   0.0000000
!  overlapFraction( 4 ,:)=  -1.8089358  -1.5934241  -2.2400966   0.0000000  -5.1946913E-08  -1.9011592
!  overlapFraction( 5 ,:)=  -2.1573510  -0.7335154  -1.9327564  -8.8930570E-08   0.0000000  -1.3187480
!  overlapFraction( 6 ,:)=  -5.0655699   7.1528341E-08   0.0000000 -10.4712095  -4.2427635   0.0000000
!  iOfMinMax = 6
!  srcRadiusEffective =   6.0000000E+02   2.4808350E+02   2.9373630E+02   5.8747626E+02   3.4316183E+02   1.0666254E+02
!  separations( 1 ,:)=   0.0000000   1.5982455E+03   1.1734453E+03   2.2501831E+03   1.6834824E+03   1.2469691E+03
!  separations( 2 ,:)=   1.5982455E+03   0.0000000   6.5069513E+02   1.7716586E+03   8.4295984E+02   3.5474603E+02
!  separations( 3 ,:)=   1.1734453E+03   6.5069513E+02   0.0000000   2.1972161E+03   1.3001464E+03   4.0039883E+02
!  separations( 4 ,:)=   2.2501831E+03   1.7716586E+03   2.1972161E+03   0.0000000   9.3063812E+02   1.8110247E+03
!  separations( 5 ,:)=   1.6834824E+03   8.4295984E+02   1.3001464E+03   9.3063812E+02   0.0000000   9.0236835E+02
!  separations( 6 ,:)=   1.2469691E+03   3.5474603E+02   4.0039883E+02   1.8110247E+03   9.0236835E+02   0.0000000
!  overlapFraction( 1 ,:)=   0.0000000  -1.2502700  -0.4661817  -1.7711780  -1.2338676  -0.9005110
!  overlapFraction( 2 ,:)=  -3.0238287   0.0000000  -0.4388657  -3.7733216  -1.0146363   3.0753334E-08   ***
!  overlapFraction( 3 ,:)=  -0.9522453  -0.3706567   0.0000000  -4.4802213  -2.2579715   0.0000000       ***
!  overlapFraction( 4 ,:)=  -1.8089358  -1.5934241  -2.2400966   0.0000000  -5.1946913E-08  -1.9011592
!  overlapFraction( 5 ,:)=  -2.1573510  -0.7335154  -1.9327564  -8.8930570E-08   0.0000000  -1.3187480
!  overlapFraction( 6 ,:)=  -5.0655699   7.1528341E-08   0.0000000 -10.4712095  -4.2427635   0.0000000
                                         ! ***             ***
!  iOfMinMax= 2  jOfMinMax= 6  overlapFrac=   3.0753334E-08
!  iOfMinMax= 2  jOfMinMax= 6  overlapFrac=   0.0000000
!  iOfMinMax, jOfMinMax, noNegativeOverlaps: -99 -99 T
!  Bef:iOfMinMax= -99  jOfMinMax= -99  overlapFrac(2,6)=   0.0000000
!  Bef:iOfMinMax= -99  jOfMinMax= -99  overlapFrac(3,6)=   2.5973618E-08
!  === After findMinPosMaxOverlap:  F   7.1528341E-08
!  Aft:iOfMinMax= 3  jOfMinMax= 6  overlapFrac(2,6)=   0.0000000
!  Aft:iOfMinMax= 3  jOfMinMax= 6  overlapFrac(3,6)=   2.5973618E-08
!  I:srcRadiusEffective(, 3 )=   2.9373630E+02  =>    2.9373630E+02
!  J:srcRadiusEffective(, 6 )=   1.0666254E+02  =>    1.0666254E+02
!  iOfMinMax = 3
!  srcRadiusEffective =   6.0000000E+02   2.4808350E+02   2.9373630E+02   5.8747626E+02   3.4316183E+02   1.0666254E+02
!  separations( 1 ,:)=   0.0000000   1.5982455E+03   1.1734453E+03   2.2501831E+03   1.6834824E+03   1.2469691E+03
!  separations( 2 ,:)=   1.5982455E+03   0.0000000   6.5069513E+02   1.7716586E+03   8.4295984E+02   3.5474603E+02
!  separations( 3 ,:)=   1.1734453E+03   6.5069513E+02   0.0000000   2.1972161E+03   1.3001464E+03   4.0039883E+02
!  separations( 4 ,:)=   2.2501831E+03   1.7716586E+03   2.1972161E+03   0.0000000   9.3063812E+02   1.8110247E+03
!  separations( 5 ,:)=   1.6834824E+03   8.4295984E+02   1.3001464E+03   9.3063812E+02   0.0000000   9.0236835E+02
!  separations( 6 ,:)=   1.2469691E+03   3.5474603E+02   4.0039883E+02   1.8110247E+03   9.0236835E+02   0.0000000
!  overlapFraction( 1 ,:)=   0.0000000  -1.2502700  -0.4661817  -1.7711780  -1.2338676  -0.9005110
!  overlapFraction( 2 ,:)=  -3.0238287   0.0000000  -0.4388657  -3.7733216  -1.0146363   0.0000000
!  overlapFraction( 3 ,:)=  -0.9522453  -0.3706567   0.0000000  -4.4802213  -2.2579715   2.5973618E-08
!  overlapFraction( 4 ,:)=  -1.8089358  -1.5934241  -2.2400966   0.0000000  -5.1946913E-08  -1.9011592
!  overlapFraction( 5 ,:)=  -2.1573510  -0.7335154  -1.9327564  -8.8930570E-08   0.0000000  -1.3187480
!  overlapFraction( 6 ,:)=  -5.0655699   0.0000000   7.1528341E-08 -10.4712095  -4.2427635   0.0000000
!  iOfMinMax = 6
!  srcRadiusEffective =   6.0000000E+02   2.4808350E+02   2.9373630E+02   5.8747626E+02   3.4316183E+02   1.0666254E+02
!  separations( 1 ,:)=   0.0000000   1.5982455E+03   1.1734453E+03   2.2501831E+03   1.6834824E+03   1.2469691E+03
!  separations( 2 ,:)=   1.5982455E+03   0.0000000   6.5069513E+02   1.7716586E+03   8.4295984E+02   3.5474603E+02
!  separations( 3 ,:)=   1.1734453E+03   6.5069513E+02   0.0000000   2.1972161E+03   1.3001464E+03   4.0039883E+02
!  separations( 4 ,:)=   2.2501831E+03   1.7716586E+03   2.1972161E+03   0.0000000   9.3063812E+02   1.8110247E+03
!  separations( 5 ,:)=   1.6834824E+03   8.4295984E+02   1.3001464E+03   9.3063812E+02   0.0000000   9.0236835E+02
!  separations( 6 ,:)=   1.2469691E+03   3.5474603E+02   4.0039883E+02   1.8110247E+03   9.0236835E+02   0.0000000
!  overlapFraction( 1 ,:)=   0.0000000  -1.2502700  -0.4661817  -1.7711780  -1.2338676  -0.9005110
!  overlapFraction( 2 ,:)=  -3.0238287   0.0000000  -0.4388657  -3.7733216  -1.0146363   0.0000000
!  overlapFraction( 3 ,:)=  -0.9522453  -0.3706567   0.0000000  -4.4802213  -2.2579715   2.5973618E-08
!  overlapFraction( 4 ,:)=  -1.8089358  -1.5934241  -2.2400966   0.0000000  -5.1946913E-08  -1.9011592
!  overlapFraction( 5 ,:)=  -2.1573510  -0.7335154  -1.9327564  -8.8930570E-08   0.0000000  -1.3187480
!  overlapFraction( 6 ,:)=  -5.0655699   0.0000000   7.1528341E-08 -10.4712095  -4.2427635   0.0000000
!  iOfMinMax= 3  jOfMinMax= 6  overlapFrac=   2.5973618E-08
!  iOfMinMax= 3  jOfMinMax= 6  overlapFrac=   0.0000000
!  iOfMinMax, jOfMinMax, noNegativeOverlaps: -99 -99 T
!  Bef:iOfMinMax= -99  jOfMinMax= -99  overlapFrac(2,6)=   3.0753334E-08
!  Bef:iOfMinMax= -99  jOfMinMax= -99  overlapFrac(3,6)=   0.0000000


!     !-------------------------------------------------------------------
!     ! Cleaning up
! 
!     call release(inTab)
!     call release(inSet)

  end subroutine test_adjustOverlaps


end program testingestsrcnamesaux

